import utils

# Load and initialise the ceda cxPython library
def InitCedaPyModule():
    import os
    import ctypes
    import ctypes.util

    if utils.macOS():
        # macOS requires manually loading dependent libraries for some reason
        boostLibs = ['atomic','system','chrono','thread','filesystem','locale']
        for b in boostLibs:
            ctypes.CDLL(ctypes.util.find_library('boost_' + b))

    # Add the ceda build directory to the system PATH (so cxPython dependencies can be found).
    # Only needed for Windows.
    if utils.windows():
        os.environ['PATH'] = utils.GetLibPath() + ";" + os.environ['PATH']

    utils.LoadLibraries(__file__)

    # Initialise cxPython as an extension module
    cxPython = utils.LoadLib('cxPython')
    cxPython.InitialisePythonExt()

    # NOTE that loading executables (.exe) doesn't seem to work on Windows. When calling
    # functions that take arguments, an access violation will occur (the arguments might
    # not be marshalled propertly). For example:
    # "WindowsError: exception: access violation writing 0x0000000000036FC8)"

    # It is necessary to explictly initialise ceda libraries that are loaded using CDLL
    # so that reflection information is registered. The registration function is named
    # "Register_Project_<project name>". For example, Register_Project_cxOperation.
    # This is not needed for cxPython, providing InitialisePythonExt() is called.

def _init_ceda():
    InitCedaPyModule()

    import bootstrapceda
    rootns = bootstrapceda.GetRootNameSpace()
    return rootns

cns = _init_ceda()
del _init_ceda

ceda = cns.ceda

__all__ = ['cns','ceda']
