# namespace.py

# Import the ceda namespace
import bootstrapceda
cns = bootstrapceda.GetRootNameSpace()
ceda = cns.ceda

def PrintEnumInfo(e,indent):
    print ' '*indent + 'enum ' + `e.GetName()`
    print ' '*indent + '{'
    for i in range(0,e.GetCount()):
        str = ' '*indent + '    '
        if not e.GetEnable(i):
            str += '-'
        str += `e.GetString(i)` + ' = ' + `e.GetValue(i)`
        print str 
    print ' '*indent + '}'

def GetFunctionPrototype(rbcRet,getArgByteCode,isMethod,fn):
    proto = ''
    if rbcRet.IsNull():
        proto = 'void'
    else:
        proto = `rbcRet`
    proto += ' ' + `fn.GetName()` + '('
    for j in range(0,fn.GetNumArgs().AsInt32()):
        a = fn.GetArg(j)
        rbc = getArgByteCode(a)
        proto += `rbc` + ' ' + `a.GetName()`
        if j != fn.GetNumArgs().AsInt32()-1:
            proto += ','
    proto += ')'
    if isMethod and fn.IsConst():
        proto += ' const'
    return proto

# Returns a string representation of the prototype of an interface method
def GetInterfaceMethodPrototype(ri,m):
    rbcRet = ceda.GetInterfaceMethodReturnTypeByteCode(ri,m)
    def getArgByteCode(a):
        return ceda.GetInterfaceMethodArgByteCode(ri,a)
    isMethod = 1
    return GetFunctionPrototype(rbcRet,getArgByteCode,isMethod,m)

# Returns a string representation of the prototype of a class method
def GetClassMethodPrototype(rc,m):
    rbcRet = ceda.GetClassMethodReturnTypeByteCode(rc,m)
    def getArgByteCode(a):
        return ceda.GetClassMethodArgByteCode(rc,a)
    isMethod = 1
    return GetFunctionPrototype(rbcRet,getArgByteCode,isMethod,m)

def GetGlobalFunctionPrototype(gf):
    rbcRet = ceda.GetGlobalFunctionReturnTypeByteCode(gf)
    def getArgByteCode(a):
        return ceda.GetGlobalFunctionArgByteCode(gf,a)
    isMethod = 0
    return GetFunctionPrototype(rbcRet,getArgByteCode,isMethod,gf)
    
def GetFunctorPrototype(f):
    rbcRet = ceda.GetFunctorReturnTypeByteCode(f)
    def getArgByteCode(a):
        return ceda.GetFunctorArgByteCode(f,a)
    isMethod = 0
    return GetFunctionPrototype(rbcRet,getArgByteCode,isMethod,f)
    
def PrintGlobalFunctionInfo(gf,indent):
    print ' '*indent + 'global function ' + GetGlobalFunctionPrototype(gf)

def PrintFunctorInfo(f,indent):
    print ' '*indent + 'functor ' + GetFunctorPrototype(f)

def PrintGlobalVariableInfo(gv,indent):
    rbc = ceda.GetGlobalVariableByteCode(gv)
    print ' '*indent + 'global variable ' + `rbc` + ' ' + `gv.GetName()`

def PrintTypedefInfo(t,indent):
    rbc = ceda.GetTypeDefByteCode(t)
    print ' '*indent + 'typedef ' + `rbc` + ' ' + `t.GetName()`

def PrintInterfaceInfo(ri,indent):
    print ' '*indent + 'interface ' + `ri.GetName()`
    print ' '*indent + '{'
    for i in range(0,ri.GetNumMethods().AsInt32()):
        m = ri.GetMethod(i)
        print ' '*indent + '    ' + GetInterfaceMethodPrototype(ri,m)
    print ' '*indent + '}'

def PrintVariantInfo(rv,indent):
    print ' '*indent + 'variant ' + `rv.GetName()`
    print ' '*indent + '{'
    for i in range(0,rv.GetNumFields().AsInt32()):
        f = rv.GetField(i)
        rbc = ceda.GetVariantFieldByteCode(rv,f)
        str = '[' + `f.GetOffset()` + ']'
        str += '    '
        str += `rbc` + ' ' + `f.GetName()`
        print ' '*indent + '    ' + str
    print ' '*indent + '}'
    
def GetClassSpecifier(c):
    #if c.IsStruct():
        #return 'datasource'
    if c.IsModel():
        return 'model'
    elif c.IsAdt():
        return 'adt'
    elif c.IsStruct():
        return 'struct'
    else:
        return 'class'

def PrintClassInfo(c,indent):
    print ' '*indent + GetClassSpecifier(c) + ' ' + `c.GetName()`
    print ' '*indent + '{'
    indent += 4
    #print ' '*indent + 'Flags = ' + `c.GetFlags()`
    print ' '*indent + 'Size = ' + `c.GetSize()` + ' bytes'
    print ' '*indent + 'CanCreate = ' + `c.CanCreate()`
    
    if c.GetNumModelFields() > 0:
        print ' '*indent + 'model'
        print ' '*indent + '{'
        for i in range(0,c.GetNumModelFields().AsInt32()):
            f = c.GetModelField(i)
            rbc = ceda.GetModelFieldByteCode(c,f)
            str = '[' + `f.GetOffset()` + ']'
            str += '    '
            str += `rbc` + ' ' + `f.GetName()` + '    '
            str += '[' + `f.GetRsn1()` + ',' + `f.GetRsn2()` + ')'
            if not f.Present():
                str += '*'
            print ' '*indent + '    ' + str
        print ' '*indent + '}'

    if c.GetNumFields() > 0:
        print ' '*indent + 'fields'
        print ' '*indent + '{'
        for i in range(0,c.GetNumFields().AsInt32()):
            f = c.GetField(i)
            rbc = ceda.GetFieldByteCode(c,f)
            str = '[' + `f.GetOffset()` + ']'
            str += '    '
            str += `rbc` + ' ' + `f.GetName()`
            print ' '*indent + '    ' + str
        print ' '*indent + '}'  

    if c.GetNumMethods() > 0:
        print ' '*indent + 'methods'
        print ' '*indent + '{'
        for i in range(0,c.GetNumMethods().AsInt32()):
            m = c.GetMethod(i)
            print ' '*indent + '    ' + GetClassMethodPrototype(c,m)
        print ' '*indent + '}'  

    if c.GetNumIndirectInterfaces() > 0:
        print ' '*indent + 'indirect interfaces'
        print ' '*indent + '{'
        for i in range(0,c.GetNumIndirectInterfaces().AsInt32()):
            nm = c.GetInterfaceName(i)
            print ' '*indent + '    ' + `nm`
        print ' '*indent + '}'  

    if c.GetNumDirectInterfaces() > 0:
        print ' '*indent + 'direct interfaces'
        print ' '*indent + '{'
        for i in range(c.GetNumIndirectInterfaces().AsInt32(),c.GetNumInterfaces().AsInt32()):
            nm = c.GetInterfaceName(i)
            print ' '*indent + '    ' + `nm`
        print ' '*indent + '}'  
    
    indent -= 4
    print ' '*indent + '}'

def PrintNameSpaceElement(nse,indent):
    tp = nse.GetType()
    if tp == ceda.ENSType.NST_Interface:
        PrintInterfaceInfo(nse.AsInterface(),indent)
    elif tp == ceda.ENSType.NST_Class:
        PrintClassInfo(nse.AsClass(),indent)
    elif tp == ceda.ENSType.NST_GlobalFunction:
        PrintGlobalFunctionInfo(nse.AsGlobalFunction(),indent)
    elif tp == ceda.ENSType.NST_GlobalVariable:
        PrintGlobalVariableInfo(nse.AsGlobalVariable(),indent)
    elif tp == ceda.ENSType.NST_Typedef:
        PrintTypedefInfo(nse.AsTypedef(),indent)
    elif tp == ceda.ENSType.NST_Enum:
        PrintEnumInfo(nse.AsEnum(),indent)
    elif tp == ceda.ENSType.NST_Functor:
        PrintFunctorInfo(nse.AsFunctor(),indent)
    elif tp == ceda.ENSType.NST_Variant:
        PrintVariantInfo(nse.AsVariant(),indent)
    else:
        print 'unrecognised namespace element type ' + `tp`
        Assert(False)

def DumpNameSpace(ns,indent):
    # Show all elements in namespace ns
    i = ns.GetElementIterator()
    while not i.AtEnd():
        nse = i.GetElement()
        tp = nse.GetType()
        PrintNameSpaceElement(nse,indent)
        i.Next()
    i.Close()

    # Recurse into child namespaces
    i = ns.GetChildNameSpaceIterator()
    while not i.AtEnd():
        print ' '*indent + 'namespace ' + `i.GetName()`
        DumpNameSpace(i.GetNameSpace(),indent+4)
        i.Next()
    i.Close()
