# pstore.py

from . import ceda

class Txn:
    def __enter__(self):
        self._cspace = ceda.GetThreadCSpace()
        self._cspace.Lock(ceda.ECSpaceLockMode.Transaction)
    def __exit__(self, type, value, traceback):
        self._cspace.Unlock()
        self._cspace = None

# Opens the persistent store at the given path and returns it.
def openStore(path, mode):
    try:
        s = ceda.PersistStoreSettings()
        s.lssSettings.flushTimeMilliSec = 500
        print 'Opening store', path
        pstore = ceda.OpenPersistStore(path, mode, s)
    except RuntimeError, e:
        print e
        raise e
    return pstore

# Opens the pspace named 'name' in the given pstore, and sets the pspace and corresponding
# cspace in thread local storage. The pspace is returned. 
# If the cspace is required by the caller, ceda.GetThreadCSpace can be called.
def openPSpace(pstore, name, mode):
    print 'Opening pspace'
    pspace = ceda.OpenPSpace(pstore, name, mode, None)
    if pspace is not None:
        cspace = pspace.GetCSpace()
        if cspace is None:
            raise RuntimeError('cspace is null')
    else:
        raise RuntimeError('pspace is null')
    ceda.SetThreadPSpace(pspace)
    ceda.SetThreadCSpace(cspace)
    return pspace

